import aiohttp


from .queries import get_order_query, get_orders_history_query, get_balance_query


async def get_order(self, order_id: str, session: aiohttp.ClientSession):

    query_str = get_order_query(order_id)
    payload = {"query": query_str, "variables": {}}
    response = await self.request("POST", "graphql", session, payload)

    return response


async def get_orders_history(self, page_id: str, session: aiohttp.ClientSession):
    query_str = get_orders_history_query(page_id=page_id)
    payload = {"query": query_str, "variables": {}}
    response = await self.request(
        "POST", "graphql", session, payload
    )

    return response


async def get_balance(self, session: aiohttp.ClientSession):
    query_str = get_balance_query()
    payload = {"query": query_str, "variables": {}}
    response = await self.request("POST", "graphql", session, payload)
    return response



CURRENCIES = ["USD", "EUR", "CLP"]

STABLE_COINS = ["DAI", "USDT"]

COINS = [
    "BTC",
    "ETH",
    "LTC",
    "XRP",
    "BCH",
    "TRX",
    "DASH",
    "XLM",
    "EOS",
    "DOT",
]

BALANCE_COINS = [*COINS, *STABLE_COINS, *CURRENCIES]


KRAKEN_MINIMUM = {
    "AAVE": 0.05,
    "ALGO": 15,
    "ANT": 2,
    "REP": 0.3,
    "REPV2": 0.3,
    "BAL": 0.3,
    "BAT": 30,
    "XXBT": 0.0002,
    "XBT": 0.0002,
    "BCH": 0.02,
    "ADA": 25,
    "LINK": 0.5,
    "COMP": 0.05,
    "ATOM": 1,
    "CRV": 10,
    "DAI": 5,
    "DASH": 0.05,
    "EOS": 2.5,
    "XETH": 0.005,
    "ETH": 0.005,
    "ETH2.S": 0.02,
    "XLTC": 0.05,
    "LTC": 0.05,
    "XXRP": 20,
    "XRP": 20,
    "XXLM": 20,
    "XLM": 20,
    "USDT": 5,
    "TRX": 250,
    "DOT": 0.2,
}

MINIMUM_TO_TRADE = {"kraken": KRAKEN_MINIMUM}

CEROS = {
    "USDT": 8,
    "BTC": 8,
    "DAI": 8,
    "ETH": 8,
    "BNB": 8,
    "LTC": 8,
    "BCH": 8,
    "CHA": 8,
    "LUK": 8,
    "DASH": 8,
    "XLM": 7,
    "XRP": 6,
    "TRX": 6,
    "DOT": 8,
    "CLP": 0,
    "EOS": 4,
    "USD": 2,
    "EUR": 2,
    "POL": 4,
}


KRAKEN_RENAME_COINS = {
    "XXBT": "BTC",
    "XBT": "BTC",
    "XETH": "ETH",
    "ETH": "ETH",
    "DASH": "DASH",
    "ZUSD": "USD",
    "USD": "USDT",
    "USDT": "USDT",
    "XXRP": "XRP",
    "XRP": "XRP",
    "XXLM": "XLM",
    "XLM": "XLM",
    "XLTC": "LTC",
    "LTC": "LTC",
    "DAI": "DAI",
    "BCH": "BCH",
    "TRX": "TRX",
    "EOS": "EOS",
    "ZEUR": "EUR",
    "PAXG": "PAX",
    "DOT": "DOT",
    "USDC": "USDC",
}

KRAKEN_RENAME_PAIRS = {
    "XXLMZUSD": "XLM/USD",
    "EOSUSDT": "EOS/USDT",
    "TRXUSD": "TRX/USDT",
    "XRPUSDT": "XRP/USDT",
    "XBTUSDT": "BTC/USDT",
    "ETHUSDT": "ETH/USDT",
    "DASHUSD": "DASH/USD",
    "XLTCUSDT": "LTC/USDT",
    "LTCUSDT": "LTC/USDT",
    "BCHUSDT": "BCH/USDT",
    "USDTZUSD": "USDT/USD",
    "DAIUSDT": "DAI/USDT",
    "XXBTZUSD": "BTC/USD",
    "XETHZUSD": "ETH/USD",
    "PAXGUSD": "PAX/USDT",
    "DOTUSDT": "DOT/USDT",
    "EOSUSD": "EOS/USD",
    "DAIUSD": "DAI/USD",
    "USDTEUR": "USDT/EUR",
    "XXBTZEUR": "BTC/EUR",
    "USDCUSD": "USDC/USD",
    "USDCUSDT": "USDC/USDT",
}

KRAKEN_RENAME_COINS_INV = {v: k for k, v in KRAKEN_RENAME_COINS.items()}

ORIONX_RENAME_COINS = {
    "BTC": "BTC",
    "ETH": "ETH",
    "DASH": "DASH",
    "DAI": "DAI",
    "USDT": "USDT",
    "LTC": "LTC",
    "XRP": "XRP",
    "BCH": "BCH",
    "TRX": "TRX",
    "XLM": "XLM",
    "EOS": "EOS",
}

ORIONX_RENAME_COINS_INV = {v: k for k, v in ORIONX_RENAME_COINS.items()}

LOANS_RENAME_COINS = {
    "BTC": "BTC",
    "ETH": "ETH",
    "DASH": "DASH",
    "DAI": "DAI",
    "USDT": "USDT",
    "LTC": "LTC",
    "XRP": "XRP",
    "BCH": "BCH",
    "TRX": "TRX",
    "XLM": "XLM",
    "EOS": "EOS",
}

LOANS_RENAME_COINS_INV = {v: k for k, v in LOANS_RENAME_COINS.items()}

BINANCE_RENAME_COINS = {"BTC": "BTC"}

BINANCE_RENAME_COINS_INV = {v: k for k, v in BINANCE_RENAME_COINS.items()}

RENAME_COINS = {
    "kraken": KRAKEN_RENAME_COINS,
    "orionX": ORIONX_RENAME_COINS,
    "loans": LOANS_RENAME_COINS,
    "binance": BINANCE_RENAME_COINS,
}

RENAME_COINS_INV = {
    "kraken": KRAKEN_RENAME_COINS_INV,
    "orionX": ORIONX_RENAME_COINS_INV,
    "loans": LOANS_RENAME_COINS_INV,
    "binance": BINANCE_RENAME_COINS_INV,
}

KRAKEN_FEE = 0.0016



import asyncio
import logging
import aiohttp
import hashlib
import hmac
import time
import json

from abc import ABC, abstractmethod
from dataclasses import dataclass, Field


@dataclass
class AbstractAPI(ABC):

    api_url: str
    api_key: str
    secret_key: str
    exchange: str

    def __post_init__(self):
        logging.info(
            f"EXCHANGE CLIENT: Credentials for {self.exchange} are ready.")

    async def request(
        self, method: str, path: str, session: aiohttp.ClientSession, payload: dict
    ):
        try:
            timestamp = time.time()
            data = self.get_payload(payload)
            signature = self.get_signature(
                payload=payload, timestamp=timestamp
            )
            headers = self.get_headers(signature, timestamp)
            url = self.api_url + path

            async with session.request(
                method=method,
                url=url,
                headers=headers,
                data=data,
            ) as resp:
                resp.raise_for_status()
                response = await resp.json()
                return self.parse_response(response)
        except Exception as err:
            logging.debug(err)
            raise err

    async def get_public(self, path: str, session: aiohttp.ClientSession, params: dict):
        try:
            url = self.api_url + path
            async with session.get(url=url, params=params) as resp:
                url = resp.url
                response = await resp.json()
                return response
        except Exception as err:
            logging.debug(err)
            raise err

    @abstractmethod
    def get_headers(self, signature):
        pass

    @abstractmethod
    def get_signature(self, payload, timestamp):
        pass

    @abstractmethod
    def get_payload(self, payload):
        pass

    @abstractmethod
    def parse_response(self, response):
        pass


@dataclass
class OrionXAPI(AbstractAPI):
    exchange: str = "orionX"

    def get_headers(self, signature, timestamp):
        headers = {
            "Content-Type": "application/json",
            "X-ORIONX-TIMESTAMP": str(int(timestamp)),
            "X-ORIONX-APIKEY": self.api_key,  # API Key
            "X-ORIONX-SIGNATURE": signature,
        }
        return headers

    def get_signature(self, payload, timestamp):
        body = json.dumps(payload)
        key = bytearray(self.secret_key, "utf-8")
        msg = str(int(timestamp)) + str(body)
        msg = msg.encode("utf-8")
        signature = str(hmac.HMAC(key, msg, hashlib.sha512).hexdigest())
        return signature

    def get_payload(self, payload):
        return json.dumps(payload)

    def parse_response(self, response):
        return response
